# adaptive-formants-procrustes.praat
# 20050630 djmw
# 20050722 Latest modification

# any vale <> 0: subsequent vowels are really from different speakers
special_randomization = 1

form What to do?
  choice What 1
  button Speaker centroids men women
  button Get homogeneties
  button Draw procrustes men women
  button Test blocked vs mixed
  button Draw blocked vs mixed results
  button Draw adaptation path
  button One bm test men men
endform

a = 1/2.54
nl$ = "'newline$'"
nl2$ ="'nl$''nl$'"

savedir$ = "/home/david/tex/thesis/anorm/"
filename$ = savedir$+"adaptive-formants-procrustes"
file$ = "'filename$'" + ".txt"
tablefile$ = "'filename$'"+".TableOfReal"

procedure prepare_file file$
  filedelete 'file$'
  datum$ = date$()
  fileappend 'file$' 'datum$''nl2$'
  fileappend 'file$' 'tab$'Script: adaptive-formants-procrustes.praat'nl2$'
endproc

Create TableOfReal (Pols 1973)... no
Rename... m50
men = selected ("TableOfReal")
Formula... log10(self)
Create TableOfReal (Van Nierop 1973)... no
Rename... w25
women = selected ("TableOfReal")
Formula... log10(self)

select men
plus women
Append
Rename... m50w25
m50w25 = selected ("TableOfReal")

call get_procrustes men women
wtom = selected ("Procrustes", 1)
mtow = selected ("Procrustes", 2)

call ptransform wtom women
wtomt = selected ("TableOfReal")
Rename... w25p

call ptransform mtow men
mtowt = selected ("TableOfReal")
Rename... m50p

select men
To Discriminant
dism = selected ("Discriminant")

select women
To Discriminant
disw = selected ("Discriminant")

if what = 1
  call investigate_mw_speaker_centroids men women
elsif what = 2
  # Get distances between Procrustes transformed and reference
  call get_homogeneties
elsif what = 3
  call draw_procrustes men women
elsif what = 4
  call test_blocked_vs_mixed
elsif what = 5
  call draw_adaptation_results
elsif what = 6
  call draw_adaptation_path
elsif what = 7
  call one_test_men_men
endif

exit

procedure test_blocked_vs_mixed
  call prepare_file 'file$'
  call tests1
  call procrustestests
endproc

procedure procrustestests
  fileappend 'file$' call procrustestests'nl2$'
  call classify_individual_procrustes men men
  call classify_individual_procrustes men women
  call classify_individual_procrustes women women
  call classify_individual_procrustes women men
endproc

procedure classify_individual_procrustes tl tt
  call get_individual_procrustes_transformed_table tl tt
  tt2 = selected ("TableOfReal")
  select tl
  To Discriminant
  dl = selected ("Discriminant")
  call get_fc dl tt 0 0
  fc1$ = fixed$ (fc, 3)
  call get_fc dl tt2 0 0
  fc2$ = fixed$ (fc, 3)
  select tl
  nametl$ = selected$ ("TableOfReal")
  select tt
  namett$ = selected$ ("TableOfReal")
  fileappend 'file$' Train: 'nametl$', test: 'namett$''nl$'
  fileappend 'file$' 'tab$' Fraction correct: 'fc1$', 'fc2$' (after Procrustes)'nl$'
  select tt2
  plus dl
  Remove
endproc

procedure one_test_men_men
  call get_fc_blocked dism men 0 0.5 50
  fcb = fc
  call get_fc_mixed dism men 0 0.5 50
  fcm = fc
  printline fcb, fcm 'fcb', 'fcm'
endproc

procedure do_one_bm_test dis_dot t_dot label2$ label3$ comment$
  select dis_dot
  name1_dot$  = selected$ ("Discriminant")
  select t_dot
  name2_dot$ = selected$ ("TableOfReal")
  fileappend 'file$' Train: 'name1_dot$', test: 'name2_dot$' 'comment$'
  ntries_dot = 50
  call fcalpha dis_dot t_dot 0.5 ntries_dot
  tr_dot = selected ("TableOfReal")
  Set column label (index)... 1 alpha
  Set column label (index)... 2 'label2$'
  Set column label (index)... 3 'label3$'
endproc

procedure tests1
  fileappend 'file$' call tests1'nl2$'

  printline 1 dism Bmm Mmm
  call do_one_bm_test dism men Bmm Mmm
  r1 = selected ("TableOfReal")

  printline 2 dism Bmw Mmw
  call do_one_bm_test dism women Bmw Mmw
  r2 = selected ("TableOfReal")

  printline 3 dism Bmwp Mmwp
  call do_one_bm_test dism wtomt Bmwp Mmwp (Procrustes)
  r3 = selected ("TableOfReal")

  printline 4 disw Bww Mww
  call do_one_bm_test disw women Bww Mww
  r4 = selected ("TableOfReal")

  printline 5 disw Bwm Mwm
  call do_one_bm_test disw men Bwm Mwm
  r5 = selected ("TableOfReal")

  printline 6 disw Bwmp Mwmp
  call do_one_bm_test disw mtowt Bwmp Mwmp (Procrustes)
  r6 = selected ("TableOfReal")
  
  printline 7 dism Bms75 Mms75
  call do_one_bm_test dism m50w25 Bms75 Mms75 (M50+W25)
  r7 = selected ("TableOfReal")

  printline 8 disw Bws75 Mws75
  call do_one_bm_test disw m50w25 Bws75 Mws75 (M50+W25)
  r8 = selected ("TableOfReal")

  select r1
  plus r2
  plus r3
  plus r4
  plus r5
  plus r6
  plus r7
  plus r8
  Append columns
  Rename... results_tests1
  results_tests1 = selected ("TableOfReal")
  Write to short text file... 'tablefile$'
  select r1
  plus r2
  plus r3
  plus r4
  plus r5
  plus r6
  plus r7
  plus r8
  Remove
endproc

procedure draw_one_adaptation_result col_doar texttop$
  Set column label (index)... col_doar-1 Step size \al
  Set column label (index)... col_doar Fraction correct
  Draw scatter plot... col_doar-1 col_doar 0 0 0 0 0.55 0.80 8 n + y
  Draw scatter plot... col_doar-1 col_doar+1 0 0 0 0 0.55 0.80 8 n 0 n
  Text top... n 'texttop$'
  baseline = Get value... 1 col_doar
  baseline$ = fixed$ (baseline, 3)
  One mark left... 'baseline$' y y y
  Marks bottom every... 1 0.1 n y n
  Marks left every... 1 0.05 n y n
  One mark bottom... 0.5 y n n
endproc

procedure draw_adaptation_results

  Read from file... 'tablefile$'
  tab_dar = selected ("TableOfReal")
  a = 1/2.54
  x0 = 1.65
  y0 = 0.60
  width = 4
  height = 2.45
  
  Erase all
  Font size... 10
  vpx1 = x0 * a
  vpx2 = vpx1 + width * a
  vpy1 = y0 * a
  vpy2 = vpy1 + height * a
  Select inner viewport... vpx1 vpx2 vpy1 vpy2
  call draw_one_adaptation_result 2 (a) M-M
   
  vpx1 = vpx2 + x0*a
  vpx2 = vpx1 + width * a
  Select inner viewport... vpx1 vpx2 vpy1 vpy2
  call draw_one_adaptation_result 5 (b) M-W
  
  vpx1 = x0 * a
  vpx2 = vpx1 + width * a
  vpy1 = vpy2 + y0
  vpy2 = vpy1 + height * a
  Select inner viewport... vpx1 vpx2 vpy1 vpy2
  call draw_one_adaptation_result 14 (c) W-M
  
  vpx1 = vpx2 + x0*a
  vpx2 = vpx1 + width * a
  Select inner viewport... vpx1 vpx2 vpy1 vpy2
  call draw_one_adaptation_result 11 (d) W-W

  vpx1 = x0 * a
  vpx2 = vpx1 + width * a
  vpy1 = vpy2 + y0
  vpy2 = vpy1 + height * a
  Select inner viewport... vpx1 vpx2 vpy1 vpy2
  call draw_one_adaptation_result 17 (e) W-M__p_

  vpx1 = vpx2 + x0*a
  vpx2 = vpx1 + width * a
  Select inner viewport... vpx1 vpx2 vpy1 vpy2
  call draw_one_adaptation_result 8 (f) M-W__p_

  vpx1 = x0 * a
  vpx2 = vpx1 + width * a
  vpy1 = vpy2 + y0
  vpy2 = vpy1 + height * a
  Select inner viewport... vpx1 vpx2 vpy1 vpy2
  call draw_one_adaptation_result 20 (g) M-MW
  
  vpx1 = vpx2 + x0*a
  vpx2 = vpx1 + width * a
  Select inner viewport... vpx1 vpx2 vpy1 vpy2
  call draw_one_adaptation_result 23 (h) W-MW
  
  
  vpx1 = x0 * a
  vpx2 = (x0 + width + x0 + width) * a
  vpy1 = y0 * a
  Select inner viewport... vpx1 vpx2 vpy1 vpy2
# Outer viewport:
# x: (5.032152230971128 - 0.06627296587926503)*2.54 =  12.61cm
# y: (6.283377077865267 + 0.152668416447)*2.54 = 16.34cm

  printline width x height = 12.7 x 15.8 cm
  call save m50-w25-blocked-mixed_results
endproc

procedure fcalpha dis_fca tab_fca prob_fca ntries_fca
  nalpha_fca = 21
  dalpha_fca = 0.05
  alphamin_fca = 0
  Create TableOfReal... tr nalpha_fca 3
  tresult_fca = selected ("TableOfReal")
  for ialpha to nalpha_fca
    alf_fca = alphamin_fca + (ialpha-1) * dalpha_fca
    call get_fc_blocked dis_fca tab_fca alf_fca prob_fca ntries_fca
    fcb_fca = fc_gfcb
    call   get_fc_mixed dis_fca tab_fca alf_fca prob_fca ntries_fca
    fcm_fca = fc_gfcm
    fcb$ = fixed$ (fcb_fca, 3)
    fcm$ = fixed$ (fcm_fca, 3)
    alf$ = fixed$ (alf_fca, 3)
    fileappend 'file$' 'tab$'alpha = 'alf$' fcb = 'fcb$' fcm = 'fcm$' ('ntries_fca' tries)'nl$'
    select tresult_fca
    Set value... ialpha 1 alf_fca
    Set value... ialpha 2 fcb_fca
    Set value... ialpha 3 fcm_fca
  endfor
  select tresult_fca
endproc

procedure get_fc_blocked dis_gfcb tab_gfcb alpha_gfcb prob_gfcb ntries_gfcb
  fc_gfcb = 0
  for k1 to ntries_gfcb
    call randomize_formant_table_blocked tab_gfcb
    tabr_gfcb = selected ("TableOfReal")
    call get_fc dis_gfcb tabr_gfcb alpha_gfcb prob_gfcb
    fc_gfcb += fc
    select tabr_gfcb
    Remove
  endfor
  fc_gfcb = fc_gfcb / ntries_gfcb
endproc

procedure get_fc_mixed dis_gfcm tab_gfcm alpha_gfcm prob_gfcm ntries_gfcm
  fc_gfcm = 0
  for k2 to ntries_gfcm
    call randomize_formant_table_mixed tab_gfcm
    tabr_gfcm = selected ("TableOfReal")
    call get_fc dis_gfcm tabr_gfcm alpha_gfcm prob_gfcm
    fc_gfcm += fc
    select tabr_gfcm
    Remove
  endfor
  fc_gfcm = fc_gfcm / ntries_gfcm
endproc

procedure get_fc dis_gfc tab_gfc alpha_gfc prob_gfc
  select dis_gfc
  plus tab_gfc
  To ClassificationTable (dw)... y y alpha_gfc prob_gfc
  ct_gfc = selected ("ClassificationTable")
  ttmp_gfc = selected ("TableOfReal")
  select ct_gfc
  To Confusion
  fc = Get fraction correct
  plus ttmp_gfc
  plus ct_gfc
  Remove
endproc

# Get procrustes transforms t2tot1 and t1tot2
procedure get_procrustes t1_gp t2_gp
  select t1_gp
  To TableOfReal (means by row labels)... no
  t1m_gp = selected ("TableOfReal")
  select t2_gp
  To TableOfReal (means by row labels)... no
  t2m_gp = selected ("TableOfReal")
  plus t1m_gp
  To Configuration
  c1_gp = selected ("Configuration", 1)
  c2_gp = selected ("Configuration", 2)
  To Procrustes... n
  t2tot1 = selected ("Procrustes")
  select c1_gp
  Copy... c1c
  t1c_gp = selected ("Configuration")
  plus c2_gp
  To Procrustes... n
  t1tot2 = selected ("Procrustes")
  select t1m_gp
  plus t1c_gp
  plus t2m_gp
  plus c1_gp
  plus c2_gp
  plus t1m_gp
  Remove
  
  select t2tot1
  plus t1tot2
endproc

# Transforma table with procrustes
# Result: TableOfReal with id == tp
procedure ptransform p_p tab_p
  select tab_p
  To Configuration
  ct1_p = selected ("Configuration")
  plus p_p
  To Configuration
  ct2_p = selected ("Configuration")
  To TableOfReal
  tp_p = selected ("TableOfReal")
  select ct1_p
  plus ct2_p
  Remove
  select tp_p
endproc

# make average of tableref
# per speaker Procrustus transform of tablefull
# Result: selected TableOfReal
procedure get_individual_procrustes_transformed_table tref_gipt tfull_gipt
  select tref_gipt
  To TableOfReal (means by row labels)... no
  Rename... trefmeans
  tm_gipt = selected ("TableOfReal")
  Sort by label... 0 0
  To Configuration
  cm_gipt = selected ("Configuration")
  select tfull_gipt
  nrows = Get number of rows
  ncols = Get number of columns
  Create TableOfReal... result 1 ncols
  result_gipt = selected ("TableOfReal")
  nv = 12
  nspr = nrows / nv
  # initialize means
  call get_distance_statistics trefmeans dymmy dummy -1
  for ispr to nspr
    select tfull_gipt
    irowb = 1 + (ispr - 1) * nv
    irowe = irowb + nv - 1
    Extract row ranges... 'irowb':'irowe'
    part_gipt = selected ("TableOfReal")
    Rename... speakermeans_b'ispr'
    Sort by label... 0 0
    To Configuration
    cpart_gipt = selected ("Configuration")
    plus cm_gipt
    To Procrustes... n
    p_gipt = selected ("Procrustes")
    plus cpart_gipt
    To Configuration
    ctmp_gipt = selected ("Configuration")
    To TableOfReal
    tpart_gipt = selected ("TableOfReal")
    Rename... speakermeans_a'ispr'

    call get_distance_statistics trefmeans speakermeans_b'ispr' speakermeans_a'ispr' ispr

    select result_gipt
    plus tpart_gipt
    Append
    resulta_gipt = selected ("TableOfReal")

    select tpart_gipt
    plus ctmp_gipt
    plus p_gipt
    plus cpart_gipt
    plus part_gipt
    plus result_gipt
    Remove
    result_gipt = resulta_gipt
  endfor
  select tm_gipt
  plus cm_gipt
  Remove
  select result_gipt
  Remove row (index)... 1

  call get_distance_statistics trefmeans speakermeans_b'ispr' speakermeans_a'ispr' 0
  
  select result_gipt
endproc

# Reference by name makes for faster distance calculation
procedure get_distance t1$ t2$
  select TableOfReal 't1$'
  nrowst1 = Get number of rows
  ncolst1 = Get number of columns
  distance = 0
  for irow to nrowst1
    for jcol to ncolst1      
      distance += (TableOfReal_'t1$'[irow,jcol]-TableOfReal_'t2$'[irow,jcol])^2
    endfor
  endfor
  distance = sqrt (distance) / (nrowst1 * ncolst1)
endproc

procedure get_distance_statistics tr$ tb$ ta$ ispr
  if ispr < 0
    min_dist_b = 1e38
    min_dist_a = 1e38
    max_dist_b = -1e38
    max_dist_a = -1e38
    sum_distances_b = 0
    sum_distances_a = 0
    nospr = 0
  elsif ispr > 0
    call get_distance 'tr$' 'tb$'
    dist_b = distance
    call get_distance 'tr$' 'ta$'
    dist_a = distance
    if dist_b < min_dist_b
      min_dist_b = dist_b
      min_spr_b = ispr
    endif
    if dist_a < min_dist_a
      min_dist_a = dist_a
      min_spr_a = ispr
    endif
    if dist_b > max_dist_b
      max_dist_b = dist_b
      max_spr_b = ispr
    endif
    if dist_a > max_dist_a
      max_dist_a = dist_a
      max_spr_a = ispr
    endif
    sum_distances_b += dist_b
    sum_distances_a += dist_a
    nospr += 1
  else
    # Give summary
    sum_distances_b = sum_distances_b / nospr
    sum_distances_a = sum_distances_a / nospr
    distance_b$ = fixed$ (sum_distances_b, 4)
    distance_a$ = fixed$ (sum_distances_a, 4)
    printline 'tab$'Average distance before = 'distance_b$' (nospr='nospr')
    printline 'tab$'Average distance after  = 'distance_a$' (nospr='nospr')
    printline
    printline 'tab$'Minimum individual distance before = 'min_dist_b' spr='min_spr_b'
    printline 'tab$'Minimum individual distance after  = 'min_dist_a' spr='min_spr_a'
    printline 'tab$'Maximum individual distance before = 'max_dist_b' spr='max_spr_b' 
    printline 'tab$'Maximum individual distance after  = 'max_dist_a' spr='max_spr_a'
  endif    
endproc

procedure get_homogeneties
  printline 'nl$'Men_to_men before and after Procrustes
  call get_individual_procrustes_transformed_table men men
  Rename... men_to_men
  
  printline 'nl$'Women_to_men before and after Procrustes
  call get_individual_procrustes_transformed_table men women
  Rename... women_to_men

  printline 'nl$'Men_to_women before and after Procrustes
  call get_individual_procrustes_transformed_table women men
  Rename... men_to_women

  printline 'nl$'Women_to_women before and after Procrustes
  call get_individual_procrustes_transformed_table women women
  Rename... women_to_women
endproc

procedure draw_procrustes t1 t2
  call get_procrustes t1 t2
  select t1
  To TableOfReal (means by row labels)... no
  t1mean = selected ("TableOfReal")
  select t2
  To TableOfReal (means by row labels)... no
  t2mean = selected ("TableOfReal")
  nrowsmean = Get number of rows
  call ptransform t2tot1 t2mean
  t2p = selected ("TableOfReal")
  call get_individual_procrustes_transformed_table t1 t2
  t2pi = selected ("TableOfReal")
  To TableOfReal (means by row labels)... no
  t2pim = selected ("TableOfReal")
  nrowsp1 = nrowsmean + 1
  Create TableOfReal... vector nrowsp1 4*2
  vector = selected ("TableOfReal")
  for irow to nrowsmean
    select t1mean
    label$ = Get row label... irow
    c1 = Get value... irow 1
    c2 = Get value... irow 2
    select t2mean
    c3 = Get value... irow 1
    c4 = Get value... irow 2
    select t2p
    c5 = Get value... irow 1
    c6 = Get value... irow 2
    select t2pim
    c7 = Get value... irow 1
    c8 = Get value... irow 2
    select vector
    Set row label (index)... irow 'label$'
    for j to 8
      Set value... irow j c'j'
    endfor
  endfor
  select t1mean
  c1 = Get column mean (index)... 1
  c2 = Get column mean (index)... 2
  select t2mean
  c3 = Get column mean (index)... 1
  c4 = Get column mean (index)... 2
  select t2p
  c5 = Get column mean (index)... 1
  c6 = Get column mean (index)... 2
  select t2pim
  c7 = Get column mean (index)... 1
  c8 = Get column mean (index)... 2
  select vector
  Set row label (index)...  nrowsp1 +
  for j to 8
    Set value... nrowsp1 j c'j'
  endfor

  Erase all
  Select outer viewport... 0 10*a 0 10*a
  select vector
  Black
  Draw scatter plot... 1 2 1 nrowsmean 2.4 3.1 2.8 3.5 8 y + y
  Draw scatter plot... 3 4 1 nrowsmean 2.4 3.1 2.8 3.5 12 y + n
  Draw vectors... 3 4 5 6 2.4 3.1 2.8 3.5 line 0 n
  Text... TableOfReal_vector[nrowsp1,1] Centre TableOfReal_vector[nrowsp1,2] Half M
  Text... TableOfReal_vector[nrowsp1,3] Centre TableOfReal_vector[nrowsp1,4] Half W
  Dotted line
  Draw vectors... 3 4 7 8 2.4 3.1 2.8 3.5 Line 0 n
  Plain line
  Text bottom... y log %F__1_
  Text left... y log %F__2_
  Marks bottom every... 1 0.1 n y n
  Marks left every... 1 0.1 n y n
  call save formants-procrustes

  select t1mean
  plus t2mean
  plus t2p
  plus t1tot2
  plus t2tot1
  plus t2pi
  plus t2pim
  plus vector
  Remove

endproc

procedure save gfile$
  path$ = "'savedir$'"+"'gfile$'"
  Write to EPS file... 'path$'.eps
  system epstopdf --outfile='path$'.pdf 'path$'.eps
endproc 

# mix speakers 
procedure randomize_formant_table_blocked tab_rftb
  select tab_rftb
  nrows = Get number of rows
  Create Permutation... pb nrows y
  pid_rftb = selected ("Permutation")
  Permute randomly (blocks)... 0 0 12 y n
  pidp_rftb = selected ("Permutation")
  plus tab_rftb
  Permute rows
  tabrandom_rftb = selected ("TableOfReal")
  select pid_rftb
  plus pidp_rftb
  Remove
  select tabrandom_rftb
endproc

procedure randomize_formant_table_mixed tab_rftm
  select tab_rftm
  if special_randomization = 0
    Randomize rows
    tabrandom_rftm = selected ("TableOfReal")
  else
    nrows = Get number of rows
    Create Permutation... pm nrows y
    pid_rftm = selected ("Permutation")
    Interleave... 0 0 12 1
    pidp_rftm = selected ("Permutation")
    Permute randomly (blocks)... 0 0 12 y n
    pidpp_rftm = selected ("Permutation")
    plus tab_rftm
    Permute rows
    tabrandom_rftm = selected ("TableOfReal")
    select pid_rftm
    plus pidp_rftm
    plus pidpp_rftm
    Remove
  endif
  select tabrandom_rftm
endproc

procedure investigate_mw_speaker_centroids mtab_isc wtab_isc
  call get_speaker_centroids mtab_isc m
  mc_isc = selected ("TableOfReal")
  call get_speaker_centroids wtab_isc f
  wc_isc = selected ("TableOfReal")
  plus mc_isc
  Append
  mwc_isc = selected ("TableOfReal")
  To PCA
  pca_isc = selected ("PCA")
  plus mc_isc
  To Configuration... 0
  cmc_isc = selected ("Configuration")
  select wc_isc
  plus pca_isc
  To Configuration... 0
  cwc_isc = selected ("Configuration")
  select cmc_isc
  To TableOfReal
  tcmc_isc = selected ("TableOfReal")
  To SSCP... 0 0 0 0
  sscp_tcmc = selected ("SSCP")
  select cwc_isc
  To TableOfReal
  tcwc_isc = selected ("TableOfReal")
  To SSCP... 0 0 0 0
  sscp_tcwc = selected ("SSCP")

  call draw_mw_speaker_centroids cmc_isc cwc_isc pca_isc sscp_tcmc sscp_tcwc

  select mc_isc
  To PCA
  pca_mc_isc = selected ("PCA")

  select wc_isc
  To PCA
  pca_wc_isc = selected ("PCA")
  
  call draw_separate_eigenvectors pca_mc_isc pca_wc_isc

#  select mc_isc
#  plus wc_isc
  select mwc_isc
  plus pca_isc
  plus cwc_isc
  plus cmc_isc
  plus tcmc_isc
  plus sscp_tcmc
  plus tcwc_isc
  plus sscp_tcwc
#  plus pca_mc_isc
#  plus pca_wc_isc
  Remove

endproc

procedure draw_mw_speaker_centroids cmc_dsc cwc_dsc pca_dsc sscp_tcmc_dsc sscp_tcwc_dsc
  x0 = 1.5
  y0 = 1
  x1 = -5.40
  x2 = -5.14
  y1 = -0.96
  y2 = -0.80
  vx = abs (x2 - x1)
  vy = abs (y2 - y1)
  
  Erase all
  vpx1 = x0 * a
  vpx2 = 9 * a  - vpx1
  vpy1 = y0 * a
  height = (9 - x0) * vy / vx
  vpy2 = vpy1 + height * a
  Select inner viewport... vpx1  vpx2 vpy1 vpy2
  Red
  select cwc_dsc
  Draw... 1 2 x1 x2 y1 y2 6 y + y
  Black
  select cmc_dsc
  Draw... 1 2 x1 x2 y1 y2 6 y + n
  Marks left every... 1 0.02 n y n
  Marks bottom every... 1 0.02 n y n
  select sscp_tcmc_dsc
  Draw sigma ellipse... 1 1 2 x1 x2 y1 y2 n
  select sscp_tcwc_dsc
  Red
  Draw sigma ellipse... 1 1 2 x1 x2 y1 y2 n
  Black
  Text top... n (a) Principal components

  select pca_dsc
  vpx1 = vpx2 + x0*a
  vpx2 = 12*a
  height2 = (height - 1) / 2
  vpy2 = vpy1 + height2 * a
  Select inner viewport... vpx1  vpx2 vpy1 vpy2
  Draw eigenvector... 1 n 0 0 -1 1 1 + y n
  Dotted line
  Draw eigenvector... 2 n 0 0 -1 1 1 + y n
  Plain line
  Draw inner box
  Text top... n (b) Eigenvectors
  One mark left... -1 y y n
  One mark left... 0 y y y
  One mark left... 1 n y y
  One mark bottom... 1 y y n
  One mark bottom... 2 y y n
  One mark bottom... 3 y y n 
   
  vpy1 = vpy2 + 1 * a
  vpy2 = (y0 + height) * a
  Select inner viewport... vpx1 vpx2 vpy1 vpy2
  Draw eigenvalues... 0 0 0.5 1.1 y y 1 + n
  Draw inner box
  One mark left... 0.5 y y n
  One mark left... 1 n y y 1.0
  One mark bottom... 1 y y n
  One mark bottom... 2 y y n
  One mark bottom... 3 y y n
  Text top... n (c) Eigenvalues
  Select inner viewport... x0*a vpx2 y0*a vpy2
  # 'Select outer viewport' geeft volgende dimensies
  # vp hor  5.30774278215223-0.007217847769028797 inches
  # vp vert 2.599670233528496-0.0048118985126865965 inches
  # hor x vert (cm): 13.5 x 6.6
  call save m50-w25-speaker-centroids
endproc

procedure draw_separate_eigenvectors pca_mc_dse pca_wc_dse
  Erase all
  select pca_mc_dse
  Draw eigenvector... 1 n 0 0 -1 1 1 + y y
  Dotted line
  select pca_wc_dse
  Draw eigenvector... 1 n 0 0 -1 1 1 + y n
  Plain line
  Text top... n Eigenvector 1 (plain:m50, dotted:f25)
  call save m50-w25-separate-eigenvectors
endproc

procedure get_speaker_centroids tab_gsc labelprefix$
  select tab_gsc
  name$ = selected$ ("TableOfReal")
  Copy... tab7_gsc
  nrows = Get number of rows
  for i to nrows
    sprnum = 1000 + (i-1) div 12 + 1
    Set row label (index)... i 'labelprefix$''sprnum'
  endfor
  To TableOfReal (means by row labels)... n
  Rename... 'name$'_sprcentroids
  select TableOfReal tab7_gsc
  Remove
  select TableOfReal 'name$'_sprcentroids
  nrows = Get number of rows
  slength = length (labelprefix$)
  for i to nrows
    label$ = Get row label... i
    labelnum$ = mid$ (label$,slength+1, 4)
    sprnum = 'labelnum$' - 1000
    Set row label (index)... i 'labelprefix$''sprnum'
  endfor
endproc

procedure get_relative_speaker_centroids tabref_grsc tab_grsc
  call get_speaker_centroids tab_grsc
  tabc_grsc = selected ("TableOfReal")
  call get_speaker_centroids tabref_grsc
  tabrefc_grsc = selected ("TableOfReal")
  To SSCP... 0 0 0 0
  sscp_grsc = selected ("SSCP")
  Extract centroid
  Rename... centroid
  centroid = selected ("TableOfReal")
  select tabc_grsc
  Formula... TableOfReal_centroid[1,col]-self
  
  select tabrefc_grsc
  plus sscp_grsc
  plus centroid
  Remove
  select tabc_grsc
endproc

procedure draw_one_path tab_dop nspr_dop
  n_dop = nspr_dop*12
  select tab_dop
  x1 = 0
  y1 = 0
  for i from 2 to n_dop
    x2 = Get value... i 1
    y2 = Get value... i 2
    Draw line... x1 y1 x2 y2
    x1 = x2
    y1 = y2
    if i mod 12 = 0
      ispr = (i - 1) div 12 + 1
#      Red
      Black
      Text special... x2 Centre y2 Half Helvetica 8 0 'ispr'
      Black
    endif
  endfor
  Black
endproc

procedure draw_adaptation_path
# men: y -0.04 0.04
#      x -0.06 0.02
# women y 0.02 -0.08
#       x -0.1 0.04
  call get_relative_speaker_centroids men women
  wcentre = selected ("TableOfReal")
  Rename... wcentre
  To SSCP... 0 0 0 0
  sscp_dap = selected ("SSCP")
  Extract centroid
  centroid = selected ("TableOfReal")
  Rename... wcentroid
  Erase all

  numberOfSpeakersToDraw = 10
  x0 = 1
  y0 = 1
  dx = 1
  dy = 1
  xw = 4
  yh = 4
  vpy2 = 0
  vpx2 = 0
  alpha = 0.025
  Create TableOfReal... labels 4 1
  labels = selected ("TableOfReal")
  label1$ = "(a) \al=0.025"
  label2$ = "(b) \al=0.05"
  label3$ = "(c) \al=0.1"
  label4$ = "(d) \al=0.2"
  for iplot to 4
    if (iplot mod 2) = 1
       vpx1 = x0 * a
    else
       vpx1 = vpx2 + dx * a
    endif
    vpx2 = vpx1 + xw * a
    if (iplot - 1) div 2 = 0
      vpy1 = y0 * a
    else
      vpy1 = (y0 + yh + dy) * a
    endif
    vpy2 = vpy1 + yh * a 
    Select inner viewport... vpx1 vpx2 vpy1 vpy2
    select women
    nrows = Get number of rows
    plus dism
    To ClassificationTable (dw)... y y alpha 0.5
    ct = selected ("ClassificationTable")
    mend = selected ("TableOfReal")
    select mend
    Axes... -0.1 0.04 -0.08 0.02
    call draw_one_path mend numberOfSpeakersToDraw
    Draw inner box
    Text special... 0 Centre 0 Half Helvetica 12 0 M
    wxpos = TableOfReal_wcentroid[1,1]
    wypos = TableOfReal_wcentroid[1,2]
    Text special... wxpos Centre wypos Half Helvetica 12 0 W
    Grey
    for ispr to numberOfSpeakersToDraw
      wxpos = TableOfReal_wcentre[ispr,1]
      wypos = TableOfReal_wcentre[ispr,2]
      Text special... wxpos Centre wypos Half Helvetica 10 0 'ispr'
    endfor
    Black
    labeltop$ = label'iplot'$
    Text top... n 'labeltop$'
    select ct
    plus mend
    Remove
    alpha *= 2
  endfor
  vpx1 = x0 * a
  vpx2 = (x0 + xw + dx + xw) * a
  vpy1 = y0 * a
  vpy2 = (y0 + yh + dy + yh) * a
  Select inner viewport... vpx1 vpx2 vpy1 vpy2
  call save m50-w25-adaptation_path
  select sscp_dap
  plus centroid
  Remove
  vpx = (4.520341207349081+0.1896325459317586)*2.54
  vpy = (4.3258967629046365-0.0048118985126865965)*2.54
  vpx$ = fixed$ (vpx, 2)
  vpy$ = fixed$ (vpy, 2)
  printline width x height = 'vpx$' x 'vpy$'
endproc
